% Simulink Graphtec GL800 driver
% Demo : open_system gl800ex

function gl800(block)
setup(block);

function setup(block)

block.NumInputPorts  = 0;
block.NumOutputPorts = 1;

block.SetPreCompPortInfoToDefaults;
block.SimStateCompliance = 'DefaultSimState';
block.SampleTimes = [0 0];

block.OutputPort(1).Dimensions       = 1;
block.OutputPort(1).DatatypeID  = 0; % double
block.OutputPort(1).Complexity  = 'Real';

block.NumDialogPrms     = 1;
block.DialogPrmsTunable = {'Tunable'};

block.RegBlockMethod('CheckParameters', @CheckPrms);
block.RegBlockMethod('Outputs', @Outputs);     % Required
block.RegBlockMethod('Update', @Update);
block.RegBlockMethod('Derivatives', @Derivatives);
block.RegBlockMethod('Terminate', @Terminate); % Required

function CheckPrms(block)
    param = block.DialogPrm(1).Data;
    
    if ~strcmp(class(param), 'double')
        DAStudio.error('Simulink:blocks:invalidParameter');
    end
    
%%
%% Update:
%%   Functionality    : Called to update discrete states
%%                      during simulation step
%%   Required         : No
%%   C-MEX counterpart: mdlUpdate
%%
function Update(block)

call = ['graphtecgl800.exe ' int2str(block.DialogPrm(1).Data)];

[status, stdout] = system(call);

if status > 0
    error(stdout)
else
    block.OutputPort(1).Data = double(str2double(stdout));
end

%%
%% Outputs:
%%   Functionality    : Called to generate block outputs in
%%                      simulation step
%%   Required         : Yes
%%   C-MEX counterpart: mdlOutputs
%%
function Outputs(block)
Update(block);

%%
%% Derivatives:
%%   Functionality    : Called to update derivatives of
%%                      continuous states during simulation step
%%   Required         : No
%%   C-MEX counterpart: mdlDerivatives
%%
function Derivatives(block)
Update(block);

%% Required
function Terminate(block)